@@ -1,9 +1,11 @@
+use core::time::Duration;
 use std::fs::File;
 use std::path::Path;
 
 use clap::{value_t, App, Arg};
-use parquet::file::reader::{FileReader, SerializedFileReader};
+use parquet::file::reader::{ChunkReader, FileReader, SerializedFileReader};
 use parquet::record::reader::RowIter;
+use parquet::schema::printer::print_schema;
 use rusoto_core::Region;
 use url::Url;
 
@@ -36,20 +38,44 @@ fn output_rows(iter: RowIter, offset: u32, limit: i32) {
     }
 }
 
-async fn print_json_from(source: Source, offset: u32, limit: i32) {
+fn output_thrift_schema<R: 'static + ChunkReader>(file_reader: &SerializedFileReader<R>) {
+    let parquet_metadata = file_reader.metadata();
+    print_schema(
+        &mut std::io::stdout(),
+        parquet_metadata.file_metadata().schema(),
+    );
+}
+
+async fn print_json_from(
+    source: Source,
+    offset: u32,
+    limit: i32,
+    should_output_schema: bool,
+    timeout: Duration,
+) {
     match source {
         Source::File(path) => {
             let file = File::open(&Path::new(&path)).unwrap();
             let file_reader = SerializedFileReader::new(file).unwrap();
-            output_rows(file_reader.get_row_iter(None).unwrap(), offset, limit);
+
+            if should_output_schema {
+                output_thrift_schema(&file_reader);
+            } else {
+                output_rows(file_reader.get_row_iter(None).unwrap(), offset, limit);
+            }
         }
         Source::Http(url_str) => {
             let mut reader = HttpChunkReader::new_unknown_size(url_str).await;
-            reader.start();
+            reader.start(timeout);
 
             let blocking_task = tokio::task::spawn_blocking(move || {
                 let file_reader = SerializedFileReader::new(reader).unwrap();
-                output_rows(file_reader.get_row_iter(None).unwrap(), offset, limit);
+
+                if should_output_schema {
+                    output_thrift_schema(&file_reader);
+                } else {
+                    output_rows(file_reader.get_row_iter(None).unwrap(), offset, limit);
+                }
             });
             blocking_task.await.unwrap();
         }
@@ -63,11 +89,16 @@ async fn print_json_from(source: Source, offset: u32, limit: i32) {
                 Region::default(),
             )
             .await;
-            reader.start(Region::default()).await;
+            reader.start(Region::default(), timeout).await;
 
             let blocking_task = tokio::task::spawn_blocking(move || {
                 let file_reader = SerializedFileReader::new(reader).unwrap();
-                output_rows(file_reader.get_row_iter(None).unwrap(), offset, limit);
+
+                if should_output_schema {
+                    output_thrift_schema(&file_reader);
+                } else {
+                    output_rows(file_reader.get_row_iter(None).unwrap(), offset, limit);
+                }
             });
             blocking_task.await.unwrap();
         }
@@ -77,7 +108,6 @@ async fn print_json_from(source: Source, offset: u32, limit: i32) {
 #[tokio::main]
 async fn main() {
     let matches = App::new("parquet2json")
-        .version("1.2.1")
         .about("Outputs Parquet as JSON")
         .arg(
             Arg::with_name("FILE")
@@ -85,6 +115,14 @@ async fn main() {
                 .required(true)
                 .index(1),
         )
+        .arg(
+            Arg::with_name("output_thrift_schema")
+                .short(String::from("t"))
+                .long("output_thrift_schema")
+                .value_name("BOOLEAN")
+                .help("Outputs thrift schema first")
+                .takes_value(true),
+        )
         .arg(
             Arg::with_name("offset")
                 .short(String::from('o'))
@@ -101,17 +139,49 @@ async fn main() {
                 .help("Maximum number of rows to output")
                 .takes_value(true),
         )
+        .arg(
+            Arg::with_name("timeout")
+                .short(String::from('t'))
+                .long("timeout")
+                .value_name("NUMBER")
+                .help("Request timeout in seconds")
+                .takes_value(true),
+        )
         .get_matches();
 
+    let output_thrift_schema: bool =
+        value_t!(matches, "output_thrift_schema", bool).unwrap_or(false);
     let offset: u32 = value_t!(matches, "offset", u32).unwrap_or(0);
     let limit: i32 = value_t!(matches, "limit", i32).unwrap_or(-1);
+    let timeout = Duration::from_secs(value_t!(matches, "timeout", u32).unwrap_or(60).into());
     let file: String = value_t!(matches, "FILE", String).unwrap_or_else(|e| e.exit());
 
     if file.as_str().starts_with("s3://") {
-        print_json_from(Source::S3(file), offset, limit).await;
+        print_json_from(
+            Source::S3(file),
+            offset,
+            limit,
+            output_thrift_schema,
+            timeout,
+        )
+        .await;
     } else if file.as_str().starts_with("http") {
-        print_json_from(Source::Http(file), offset, limit).await;
+        print_json_from(
+            Source::Http(file),
+            offset,
+            limit,
+            output_thrift_schema,
+            timeout,
+        )
+        .await;
     } else {
-        print_json_from(Source::File(file), offset, limit).await;
+        print_json_from(
+            Source::File(file),
+            offset,
+            limit,
+            output_thrift_schema,
+            timeout,
+        )
+        .await;
     }
 }
